import json
import requests
import numpy as np
import pandas as pd
from datetime import datetime, timedelta
from sklearn.preprocessing import StandardScaler
from data_preprocessing import normalize_data
from model_training import update_lstm_model, train_lstm_model
from settings import settings

def is_trading_hours():
    """거래 시간(예: 09:00 ~ 15:30) 여부를 확인합니다."""
    now = datetime.now()
    return now.weekday() < 5 and now.time() >= datetime.strptime("09:00", "%H:%M").time() and now.time() <= datetime.strptime("15:30", "%H:%M").time()

def is_update_time():
    """모델 업데이트가 필요한 시간(오후 6시 이후) 여부를 확인합니다."""
    now = datetime.now()
    return now.time() >= datetime.strptime("18:00", "%H:%M").time()

def get_real_time_data(ticker, date=None):
    # 기존의 get_real_time_data 코드 유지
    # ...
    pass

def get_valid_token():
    # 기존의 get_valid_token 코드 유지
    # ...
    pass

def is_token_valid(token_data):
    # 기존의 is_token_valid 코드 유지
    # ...
    pass

def refresh_token():
    # 기존의 refresh_token 코드 유지
    # ...
    pass

def create_dataset(dataset, time_step=1):
    # 기존의 create_dataset 코드 유지
    # ...
    pass

def real_time_prediction(model, scaler, ticker, time_step=60):
    # 기존의 real_time_prediction 코드 유지
    # ...
    pass

def update_real_time_model(model, scaler, ticker, time_step=60):
    real_time_data = batch_collect_data(ticker, start_date=datetime.now() - timedelta(days=7), end_date=datetime.now())
    if real_time_data is None:
        return model
    
    real_time_data = real_time_data[['stck_prpr']].rename(columns={'stck_prpr': 'close'})
    real_time_data, _ = normalize_data(real_time_data)
    scaled_real_time_data = real_time_data['close'].values.reshape(-1, 1)
    
    X_real_time, y_real_time = create_dataset(scaled_real_time_data, time_step)
    X_real_time = X_real_time.reshape(X_real_time.shape[0], X_real_time.shape[1], 1)
    
    model = update_lstm_model(pd.DataFrame(y_real_time), model, time_step)
    return model

def risk_management(predictions, threshold=0.05):
    # 기존의 risk_management 코드 유지
    # ...
    pass

def send_alert(message):
    # 기존의 send_alert 코드 유지
    # ...
    pass

def batch_collect_data(ticker, start_date, end_date):
    # 기존의 batch_collect_data 코드 유지
    # ...
    pass

if __name__ == '__main__':
    from data_preprocessing import preprocess_data
    from model_training import train_lstm_model
    from config import *

    STOCK_TICKER = "005930"  # 예시: 삼성전자 티커

    model = None
    scaler = None

    try:
        while True:
            if is_trading_hours():
                # 거래 시간 중에는 실시간 예측만 수행
                if model is None or scaler is None:
                    print("모델이 준비되지 않았습니다. 비거래 시간에 모델을 업데이트해야 합니다.")
                else:
                    real_time_predictions = real_time_prediction(model, scaler, STOCK_TICKER)
                    if real_time_predictions is not None:
                        print(real_time_predictions)
                    else:
                        print("Failed to fetch real-time predictions.")
            elif is_update_time():
                # 비거래 시간에 모델 업데이트 수행 (오후 6시 이후)
                stock_data = batch_collect_data(STOCK_TICKER, start_date=datetime.now() - timedelta(days=7), end_date=datetime.now())
                if stock_data is not None:
                    financial_data = pd.DataFrame()  # Placeholder
                    news_data = pd.DataFrame()  # Placeholder
                    economic_data = pd.DataFrame()  # Placeholder

                    # 데이터 전처리 및 모델 업데이트
                    (X_train, X_test, y_train, y_test), scaler, combined_data = preprocess_data(stock_data, financial_data, news_data, economic_data)
                    model = train_lstm_model(X_train, y_train)

                    print("모델이 성공적으로 업데이트되었습니다.")
                else:
                    print("모델 업데이트에 필요한 데이터를 가져오는 데 실패했습니다.")
            else:
                print("거래 시간 외이며 모델 업데이트 시간도 아닙니다. 잠시 대기합니다.")

            # 주기적으로 확인 (예: 1분 간격)
            time.sleep(60)

    except Exception as e:
        print(f"An error occurred: {e}")
