import logging
import pandas as pd
from datetime import datetime
from helper_functions import get_available_cash, get_owned_quantity
from websocket_client import connect_to_websocket

def generate_trading_signals(stock_data):
    stock_data['SMA_20'] = stock_data['Close'].rolling(window=20).mean()
    stock_data['SMA_50'] = stock_data['Close'].rolling(window=50).mean()
    stock_data['signal'] = 0
    stock_data.loc[20:, 'signal'] = (stock_data['SMA_20'][20:] > stock_data['SMA_50'][20:]).astype(int)
    stock_data['position'] = stock_data['signal'].diff()
    return stock_data

def execute_trades(stock_data):
    buy_signals = stock_data[stock_data['position'] == 1]
    sell_signals = stock_data[stock_data['position'] == -1]

    ws = connect_to_websocket()

    available_cash = get_available_cash()

    for index, signal in buy_signals.iterrows():
        stock_price = signal['Close']
        quantity = calculate_order_quantity(available_cash, stock_price)
        if quantity > 0:
            order = {
                'type': 'buy',
                'stock_code': signal['stock_code'],
                'price': stock_price,
                'quantity': quantity
            }
            ws.send(json.dumps(order))
            available_cash -= stock_price * quantity

    for index, signal in sell_signals.iterrows():
        stock_price = signal['Close']
        owned_quantity = get_owned_quantity(signal['stock_code'])
        if owned_quantity > 0:
            order = {
                'type': 'sell',
                'stock_code': signal['stock_code'],
                'price': stock_price,
                'quantity': owned_quantity
            }
            ws.send(json.dumps(order))

def calculate_order_quantity(available_cash, stock_price, risk_factor=0.01, max_portfolio_percent=0.05):
    max_investment = available_cash * risk_factor
    max_quantity = max_investment // stock_price

    max_portfolio_allocation = available_cash * max_portfolio_percent
    max_quantity = min(max_quantity, max_portfolio_allocation // stock_price)

    return max(1, int(max_quantity))
