import logging
import json
import requests
from datetime import datetime, timedelta
import matplotlib.pyplot as plt

class PerformanceMonitor:
    def __init__(self, log_file='performance_log.json'):
        self.log_file = log_file
        self.performance_data = self.load_performance_data()

    def load_performance_data(self):
        try:
            with open(self.log_file, 'r') as f:
                return json.load(f)
        except (FileNotFoundError, json.JSONDecodeError):
            return []

    def save_performance_data(self):
        with open(self.log_file, 'w') as f:
            json.dump(self.performance_data, f)

    def log_performance(self, date, equity_curve):
        performance_entry = {
            'date': date,
            'equity_curve': equity_curve
        }
        self.performance_data.append(performance_entry)
        self.save_performance_data()

    def plot_performance(self):
        dates = [entry['date'] for entry in self.performance_data]
        equity_curves = [entry['equity_curve'] for entry in self.performance_data]
        plt.plot(dates, equity_curves)
        plt.xlabel('Date')
        plt.ylabel('Equity Curve')
        plt.title('Performance Over Time')
        plt.show()

    def check_performance(self):
        recent_performance = self.performance_data[-5:]  # Check last 5 entries
        if len(recent_performance) < 5:
            return False  # Not enough data to decide

        avg_performance = sum(entry['equity_curve'] for entry in recent_performance) / len(recent_performance)
        return avg_performance < self.performance_data[-6]['equity_curve']  # Check if recent performance is worse than before

    def trigger_retraining(self):
        if self.check_performance():
            logging.info("Performance degraded. Triggering model retraining.")
            # Add your retraining logic here
            return True
        return False
